/**
 * Copyright (c) Members of the EGEE Collaboration. 2007-2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 *
 *  Authors:
 *  2007-
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *     <grid-mw-security@nikhef.nl> 
 *
 *
 */

#include "readbuf.h"


int readfdtobuf (int fd, unsigned char ** buffer, int buffersize)
{
    int num_of_bytes = 0;
    struct stat filestat;

    if (fd < 0)
    {
        err (EFAULT, "file descriptor failure, negative. Something wrong upstream\n");
        return -11;
    }


    if ((!buffer) || (!*buffer))
    {
        err (EFAULT, "buffer not provided or set.\n");
        return -12;
    }


    /* How big is that file again? */
    if (fstat(fd, &filestat))
    {
        err (errno, "stat on the opened file failed\n");
        return -51;
    }


    /* 
     * Read its contents into the allocated buffer in memory while making
     * sure the amound read is what we expect it to be... 
     */
    num_of_bytes = read(fd, *buffer, buffersize);
    if (filestat.st_size != num_of_bytes) 
    {
        err (EIO, "Read wrong number of bytes while reading the source proxy. Read %d bytes instead of the expected %d bytes\n", num_of_bytes, buffersize);
        return -91;
    }

    /* Need to store the size for later use during writing of the proxy. */
    return num_of_bytes;
}


